from yafowil.utils import entry_point
import logging
import os
import pkg_resources


logger = logging.getLogger('yafowil.plone')


if pkg_resources.require('Products.CMFPlone')[0].version.startswith('5.'):
    logger.info('Plone 5 detected. Load related YAFOWIL configuration')
    from . import plone5 as yafowil_plone
else:
    logger.info('Plone 4 detected. Load related YAFOWIL configuration')
    from . import plone4 as yafowil_plone


@entry_point(order=20)
def register():
    yafowil_plone.register()


@entry_point(order=20)
def configure():
    if not os.environ.get('TESTRUN_MARKER'):
        yafowil_plone.configure()



from Products.Five import BrowserView
from yafowil.base import factory
from yafowil.controller import Controller
from yafowil.utils import get_example
from yafowil.utils import get_example_names
from zExceptions import NotFound
import yafowil.loader  # nopep8  # loads registry


class ExampleResponseView(BrowserView):

    def __call__(self):
        route = self.route(self.request['URL'])
        for header in route.get('header', []):
            self.request.response.setHeader(*header)
        return route.get('body', '')


class ExampleView(BrowserView):

    @property
    def example_names(self):
        return sorted(get_example_names())

    @property
    def example(self):
        if self.example_name:
            return get_example(self.example_name)

    @property
    def example_name(self):
        if hasattr(self, '_example_name'):
            return self._example_name

    @property
    def route(self):
        if hasattr(self, '_route'):
            return self._route

    def forms(self):
        result = list()
        for part in self.example:
            widget = part['widget']
            form = factory(
                u'form',
                name='form-%s' % widget.name,
                props={
                    'action': self.example_name})
            form[widget.name] = widget
            form['submit'] = factory(
                'submit',
                props={
                    'label': 'submit',
                    'action': 'save',
                    'handler': lambda widget, data: None,
                    'class': 'submit-widget button-field context'})
            controller = Controller(form, self.request)
            result.append(controller.rendered)
        return result

    def publishTraverse(self, request, name):
        if self.example_name:
            view = ExampleResponseView(self.context, self.request)
            view.routes = dict()
            for part in self.example:
                routes = part.get('routes', [])
                if name in routes:
                    view.route = routes[name]
                    return view
            raise NotFound()
        if name not in self.example_names:
            raise NotFound()
        self._example_name = name
        return self



from Products.Five import BrowserView
from yafowil.controller import Controller
from yafowil.yaml import parse_from_YAML


class BaseForm(BrowserView):
    form = None
    action_resource = u''

    def form_action(self, widget, data):
        return '%s/%s' % (self.context.absolute_url(), self.action_resource)

    def render_form(self):
        self.prepare()
        controller = Controller(self.form, self.request)
        if not controller.next:
            return controller.rendered
        return controller.next

    def prepare(self):
        raise NotImplementedError(u"Abstract Form does not implement "
                                  u"``prepare``.")


class Form(BaseForm):

    def __call__(self):
        return self.render_form()


class YAMLBaseForm(BaseForm):
    form_template = None
    message_factory = None

    def prepare(self):
        self.form = parse_from_YAML(
            self.form_template, self, self.message_factory)


class YAMLForm(YAMLBaseForm):

    def __call__(self):
        return self.render_form()



from yafowil.base import factory
from yafowil.plone.connectors import plone_preprocessor
import os


###############################################################################
# plone5 specific resources
###############################################################################

resourcedir = os.path.join(os.path.dirname(__file__), 'resources', 'plone5')
css = [{
    'group': 'yafowil.plone.common',
    'resource': 'yafowil-fontello.css',
    'order': 10,
}]


###############################################################################
# configure factory
###############################################################################

def configure_factory():
    # set theme
    factory.theme = 'plone5'
    # selection
    factory.defaults['select.label_radio_class'] = 'radioType'
    factory.defaults['select.label_checkbox_class'] = 'checkboxType'


###############################################################################
# register macros
###############################################################################

def register_macros():
    factory.register_macro('form', 'form', {
        'form.class': 'enableUnloadProtection enableAutoFocus '
                      'enableFormTabbing edit-form',
    })
    factory.register_macro('field', 'field:plonelabel:error', {
        'field.class': 'field',
        'field.error_class': 'error',
        'error.class': 'fieldErrorBox',
        'error.message_class': 'error',
        'error.render_empty': True,
        'error.position': 'before',
    })
    factory.register_macro('button', 'field:submit', {
        'field.class': 'formControls',
        'submit.class': 'context',
    })
    factory.register_macro('array', 'array', {})
    factory.register_macro('arrayfield', 'field:plonelabel:error', {})


###############################################################################
# entry points
###############################################################################

def register():
    import common
    factory.register_global_preprocessors([plone_preprocessor])
    factory.register_theme('plone5', 'yafowil.plone', resourcedir, css=css)


def configure():
    configure_factory()
    register_macros()



from Products.CMFPlone import PloneMessageFactory as _
from yafowil.base import factory
from yafowil.common import generic_positional_rendering_helper
from yafowil.utils import UNSET
from yafowil.utils import cssid
from yafowil.utils import managedprops
from zope.component import getUtility
from zope.component.hooks import getSite
import pkg_resources


###############################################################################
# Plone TinyMCE hook
###############################################################################

HAS_TINYMCE = True
try:
    pkg_resources.get_distribution('Products.TinyMCE')
    from Products.TinyMCE.interfaces.utility import ITinyMCE
except pkg_resources.DistributionNotFound:
    HAS_TINYMCE = False


if HAS_TINYMCE:
    def tinymce_config(widget, data):
        request = data.request.zrequest
        path = request.physicalPathFromURL(request.getURL())
        context = getSite()
        # query object by path, if not found, context is site
        for i in range(1, len(path)):
            brains = context.portal_catalog(path={
                'query': '/'.join(path[:-(i)]),
                'depth': 0})
            if not brains:
                continue
            context = brains[0].getObject()
            break
        utility = getUtility(ITinyMCE)
        config = utility.getConfiguration(context=context, request=request)
        return config.replace('"', '&#34;')

    factory.defaults['richtext.title'] = tinymce_config


###############################################################################
# Plone specific label blueprint
###############################################################################

@managedprops('label', 'for', 'help', 'title')
def plone_label_renderer(widget, data):
    tag = data.tag
    label_text = widget.attrs.get('label', widget.__name__)
    if callable(label_text):
        label_text = label_text()
    label_attrs = {'class_': 'formQuestion'}
    help_text = widget.attrs['help']
    if callable(help_text):
        help_text = help_text()
    if data.tag.translate:
        label_text = data.tag.translate(label_text)
        help_text = data.tag.translate(help_text)
    if data.mode == 'edit':
        for_path = widget.attrs['for']
        if for_path:
            for_widget = widget.root
            for name in for_path.split('.'):
                for_widget = for_widget[name]
            label_attrs['for_'] = cssid(for_widget, 'input')
        else:
            label_attrs['for_'] = cssid(widget, 'input')
        if widget.attrs['title']:
            label_attrs['title'] = widget.attrs['title']
    label_contents = [label_text]
    if widget.attrs.get(widget.attrs['required_bullet_trigger'])\
            and data.mode == 'edit':
        label_contents.append(data.tag(
            'span', '&nbsp;',
            class_='required',
            title=_('required', 'Required')))
    label_contents.append(data.tag('span', help_text, class_='formHelp'))
    rendered = data.rendered is not UNSET and data.rendered or u''
    position = widget.attrs['position']
    if callable(position):
        position = position(widget, data)
    combined_label = u' '.join(label_contents)
    return generic_positional_rendering_helper(
        'label', combined_label, label_attrs, rendered, position, tag)


factory.register(
    'plonelabel',
    edit_renderers=[plone_label_renderer],
    display_renderers=[plone_label_renderer])

factory.doc['blueprint']['plonelabel'] = """\
Label for Plone blueprint.
"""

factory.defaults['plonelabel.position'] = 'before'

factory.doc['props']['plonelabel.label'] = """\
Text to be displayed as a label.
"""

factory.defaults['plonelabel.help'] = ''
factory.doc['props']['plonelabel.help'] = """\
Help text to be displayed inside label.
"""

factory.defaults['plonelabel.title'] = None
factory.doc['props']['plonelabel.title'] = """\
Title attribute of label text
"""

factory.defaults['plonelabel.for'] = None
factory.doc['props']['plonelabel.for'] = """\
Optional dottedpath of widget to be labled
"""

factory.defaults['plonelabel.required_bullet_trigger'] = 'required'
factory.doc['props']['plonelabel.required_bullet_trigger'] = """\
Attribute name which triggers rendering of required bullet. Defaults to
'required'.
"""



from Products.Five.browser.metaconfigure import resourceDirectory
from yafowil.base import factory
from yafowil.utils import get_plugin_names
from zope.interface import Interface


class IYAFOWILResourceDirective(Interface):
    """Loader for plugin resources.
    """


def yafowil_resource_directive(_context):
    """Register plugin resource directories as plone resource directories
    to be accessible via browser URL.
    """
    for plugin_name in get_plugin_names():
        resources = factory.resources_for(plugin_name)
        if not resources:
            continue
        resourceDirectory(_context, plugin_name, resources['resourcedir'])



from Products.Five import BrowserView
from StringIO import StringIO
from operator import itemgetter
from plone.registry.interfaces import IRegistry
from yafowil.base import factory
from yafowil.utils import get_plugin_names
from zope.component import getUtility
import logging
import os
import sys


logger = logging.getLogger('yafowil.plone')


def enabled_resources(which, verbose=False):
    """Return enabled YAFOWIL resources.

    ``which`` is either "js" or "css".
    """
    result = list()
    registry = getUtility(IRegistry)
    for plugin_name in get_plugin_names():
        resources = factory.resources_for(plugin_name)
        if not resources:
            continue
        resourcedir = resources['resourcedir']
        for record in resources[which]:
            if not registry.get(record['group']):
                verbose and logger.warning(
                    "Skipping resource '%s' for group '%s'" % (
                        record['resource'], record['group']
                    )
                )
                continue
            if 'order' not in record:
                record['order'] = sys.maxint
            if record['resource'].startswith('http'):
                record['url'] = record['resource']
                record['path'] = None
            else:
                resdirname = '++resource++%s' % plugin_name
                record['url'] = '%s/%s' % (resdirname, record['resource'])
                record['path'] = os.path.join(resourcedir, record['resource'])
            result.append(record)
            verbose and logger.info(
                "Activate resource '%s' for group '%s'" % (
                    record['resource'], record['group']
                )
            )
    return sorted(result, key=itemgetter('order'))


class Resources(BrowserView):
    """Browser view for concatenating and delivering resources defined in
    YAFOWIL resource groups.

    This code is inspired by resource delivery views in collective.js.jqueryui.

    Plone 5 only.
    """
    _header_template = u"\n/* yafowil.plone: %s */\n"
    _resource_type= None
    _mimetype = None

    def __call__(self):
        self.request.response.setHeader('Content-Type', self._mimetype)
        return self.get_resources_content(
            enabled_resources(self._resource_type)
        )

    def get_resources_content(self, resources):
        data = StringIO()
        data.write(self._header_template % (self._resource_type))
        data.write(u"\n")
        for resource in resources:
            if resource['path'] is None:
                logger.error('Remote YAFOWIL resource found which cannot be '
                             'included in bundle {}'.format(resource['url']))
                continue
            with open(resource['path'], 'r') as fd:
                content = fd.read()
            try:
                content = unicode(content)
            except Exception, e:
                content = unicode(content.decode('utf-8'))
            data.write(content)
            data.write(u"\n")
        return data.getvalue()


class YafowilJS(Resources):
    _resource_type = 'js'
    _mimetype = 'application/javascript'


class YafowilCSS(Resources):
    _resource_type = 'css'
    _mimetype = 'text/css'



from UserDict import DictMixin
from ZPublisher.HTTPRequest import FileUpload
from ZPublisher.HTTPRequest import HTTPRequest
from zope.i18n import translate
from zope.i18nmessageid import Message


class Zope2RequestAdapter(DictMixin):

    def __init__(self, request):
        if isinstance(request, self.__class__):
            # for some rare cases this makes sense
            self.zrequest = request.zrequest
        else:
            self.zrequest = request.REQUEST
        if not isinstance(self.zrequest, HTTPRequest):
            raise ValueError('Expecting request based on '
                             'ZPublisher.HTTPRequest.HTTPRequest')

    def __getitem__(self, key):
        value = self.zrequest.form[key]
        if isinstance(value, FileUpload):
            fvalue = dict()
            fvalue['file'] = value
            fvalue['filename'] = value.filename
            fvalue['mimetype'] = value.headers.get('content-type', '')
            fvalue['headers'] = value.headers
            fvalue['original'] = value
            return fvalue
        return value

    def keys(self):
        return self.zrequest.form.keys()

    def __setitem__(self, key, item):
        raise AttributeError('read only, __setitem__ is not supported')

    def __delitem__(self, key):
        raise AttributeError('read only, __delitem__ is not supported')


class ZopeTranslation(object):

    def __init__(self, data):
        self.zrequest = data.request.zrequest

    def __call__(self, msg):
        if not isinstance(msg, Message):
            return msg
        return translate(msg, context=self.zrequest)


def plone_preprocessor(widget, data):
    if not isinstance(data.request, (dict, Zope2RequestAdapter)):
        data.request = Zope2RequestAdapter(data.request)
    if not isinstance(data.translate_callable, ZopeTranslation):
        data.translate_callable = ZopeTranslation(data)
    return data



from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import interfaces as Plone
from Products.CMFQuickInstallerTool import interfaces as QuickInstaller
from yafowil.plone.resources import enabled_resources
from zope.interface import implementer


@implementer(Plone.INonInstallable)
class HiddenProfiles(object):

    def getNonInstallableProfiles(self):
        """Do not show on Plone's list of installable profiles.
        """
        return [
            'yafowil.plone:demo',
            'yafowil.plone:demoresources'
        ]


@implementer(QuickInstaller.INonInstallable)
class HiddenProducts(object):

    def getNonInstallableProducts(self):
        """Do not show on QuickInstaller's list of installable products.
        """
        return [
            'yafowil.plone:demo',
            'yafowil.plone:demoresources'
        ]


def setup_resource_registries(context):
    """Registers all enabled yafowil resource groups to portal_css and
    portal_javascripts.

    Plone 4 only.

    ``context`` is a Products.GenericSetup.context.DirectoryImportContext
    instance.
    """
    if not context.readDataFile('yafowil.plone.txt'):
        return
    site = context.getSite()
    msg = 'Stylesheets (CSS)'
    regcss = getToolByName(site, 'portal_css')
    for record in enabled_resources('css', verbose=True):
        merge = record.get('merge', True)
        msg += '<br />%s' % record['resource']
        if [_ for _ in regcss.getResources() if record['url'] == _.getId()]:
            msg += ' skipped'
            continue
        msg += ' registered'
        regcss.registerStylesheet(record['url'], expression='', media='screen',
            rel='stylesheet', title='', rendering='link', enabled=1,
            cookable=merge, compression='safe', cacheable=True,
            conditionalcomment='', authenticated=False, skipCooking=False,
            applyPrefix=True)
    msg += '<br /><br />Javascripts (JS)'
    regjs = getToolByName(site, 'portal_javascripts')
    for record in enabled_resources('js', verbose=True):
        merge = record.get('merge', True)
        msg += '<br />%s' % record['resource']
        if [_ for _ in regjs.getResources() if record['url'] == _.getId()]:
            msg += ' skipped'
            continue
        msg += ' registered'
        regjs.registerScript(record['url'], expression='', inline=False,
            enabled=True, cookable=merge, compression='safe', cacheable=True,
            conditionalcomment='', authenticated=False, skipCooking=False)
    return msg
